@@ -107,6 +107,7 @@ sub get_domain_owner
 		my ($dmn,$usr) = split(/: /, $_);
 		if ($dmn eq $domain)
 		{
+			close(DOMAINOWNERS);
 			return $usr;
 		}
 	}
@@ -234,22 +235,66 @@ sub check_limits
 			{
 				die("You ($name) have reach your daily email limit of $email_limit emails\n");
 			}
+
+			open(USAGE, ">>/etc/virtual/usage/$name");
+			print USAGE "1";
+			close(USAGE);
+			chmod (0660, "/etc/virtual/usage/$name");
 		}
 	}
-	
-	open(USAGE, ">>/etc/virtual/usage/$name");
-	print USAGE "1";
-	close(USAGE);
-	chmod (0660, "/etc/virtual/usage/$name");
 
-	log_bandwidth($uid);
+	my $sender_address = Exim::expand_string('$sender_address');
+	my $mid = Exim::expand_string('$message_id');
+
+	log_bandwidth($uid,"type=email&email=$sender_address&method=outgoing&id=$mid");
 
 	return "yes"
 }
 
+sub log_email
+{
+	my($lp,$dmn) = @_;
+
+	#log_str("logging $lp\@$dmn\n");
+	my $user = get_domain_owner($dmn);
+	if ($user == -1) { return "no"; }
+
+	my $mid = Exim::expand_string('$message_id');
+
+	if ( (@pw = getpwnam($user))  )
+	{
+		log_bandwidth($pw[2],"type=email&email=$lp\@$dmn&method=incoming&id=$mid");
+	}
+
+	return "yes";
+}
+
+sub save_virtual_user
+{
+	my $dmn = Exim::expand_string('$domain');
+	my $lp  = Exim::expand_string('$local_part');
+	my $usr = "";
+	my $pss = "";
+	my $entry = "";
+
+	open (PASSWD, "/etc/virtual/$dmn/passwd") || return "no";
+
+	while ($entry = <PASSWD>) {
+		($usr,$pss) = split(/:/,$entry);
+		if ($usr eq $lp) {
+			close(PASSWD);
+			log_email($lp, $dmn);
+			return "yes";
+		}
+	}
+	close (PASSWD);
+
+	return "no";
+}
+
 sub log_bandwidth
 {
-	my ($uid) = @_;
+	my ($uid,$data) = @_;
 	my $name = getpwuid($uid);
 
 	if (uid_exempt($uid)) { return; }
@@ -261,7 +306,7 @@ sub log_bandwidth
 	if ($bytes == -1) { return; }
 
 	open (BYTES, ">>/etc/virtual/usage/$name.bytes");
-	print BYTES "$bytes\n";
+	print BYTES "$bytes=$data\n";
 	close(BYTES);
 	chmod (0660, "/etc/virtual/usage/$name.bytes");
 }
